@@ -12,7 +12,8 @@
 # @synopsis
 #
 # \arguments{
-#   \item{files}{A @list of @see "GenericDataFile":s.}
+#   \item{files}{A @list of @see "GenericDataFile":s or
+#      a @see "GenericDataFileSet".}
 #   \item{tags}{A @character @vector of tags to be used for this file set.
 #      The string \code{"*"} indicates that it should be replaced by the
 #      tags part of the file set pathname.}
@@ -36,7 +37,8 @@ setConstructorS3("GenericDataFileSet", function(files=NULL, tags="*", depth=NULL
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Arguments 'files':
   if (is.null(files)) {
-  } else if (is.list(files)) {
+  } else if (inherits(files, "GenericDataFileSet") || is.list(files)) {
+    if (!is.list(files)) files <- as.list(files)
     reqFileClass <- GenericDataFileSet$getFileClass();
     base::lapply(files, FUN=function(df) {
       Arguments$getInstanceOf(df, reqFileClass)
@@ -67,10 +69,12 @@ setConstructorS3("GenericDataFileSet", function(files=NULL, tags="*", depth=NULL
     }
   }
 
+  files <- as.list(files)
+
 
   this <- extend(Object(), c("GenericDataFileSet", uses("FullNameInterface")),
     "cached:.fileSize" = NULL,
-    files = as.list(files),
+    files = files,
     .depth = depth,
     .tags = NULL
   );
@@ -154,12 +158,21 @@ setMethodS3("as.character", "GenericDataFileSet", function(x, ...) {
   s <- c(s, paste("Path (to the first file): ", path, sep=""));
 
   # File size
-  fileSizeB <- sprintf("%.2f MB", getFileSize(this, "numeric")/1024^2);
-  s <- c(s, sprintf("Total file size: %s", fileSizeB));
+  fileSize <- getFileSize(this, "units");
+  if (!is.na(fileSize)) {
+    fileSizeB <- sprintf("%.0f bytes", getFileSize(this, "numeric"));
+    if (fileSizeB != fileSize) {
+      fileSize <- sprintf("%s (%s)", fileSize, fileSizeB);
+    }
+  }
+  s <- c(s, sprintf("Total file size: %s", fileSize));
 
   # RAM
   s <- c(s, sprintf("RAM: %.2fMB", objectSize(this)/1024^2));
 
+  # Check fullnames translation
+  getFullNames(this, onRemapping="warning")
+
   GenericSummary(s);
 }, protected=TRUE)
 
@@ -252,7 +265,10 @@ setMethodS3("validate", "GenericDataFileSet", function(this, ...) {
 
 
 
-setMethodS3("getFileSize", "GenericDataFileSet", function(this, ..., force=FALSE) {
+setMethodS3("getFileSize", "GenericDataFileSet", function(this, what=c("numeric", "units"), sep="", ..., force=FALSE) {
+  # Argument 'what':
+  what <- match.arg(what);
+
   fileSize <- this$.fileSize;
   if (force || is.null(fileSize)) {
     files <- getFiles(this);
@@ -260,6 +276,24 @@ setMethodS3("getFileSize", "GenericDataFileSet", function(this, ..., force=FALSE
     fileSize <- sum(fileSizes, na.rm=TRUE);
     this$.fileSize <-  fileSize;
   }
+
+  if (what == "numeric")
+    return(fileSize);
+
+  if (is.na(fileSize))
+    return(fileSize);
+
+  units <- c("bytes", "kB", "MB", "GB", "TB");
+  scale <- 1;
+  for (kk in seq_along(units)) {
+    unit <- units[kk];
+    if (fileSize < 1000)
+      break;
+    fileSize <- fileSize/1024;
+  }
+  fileSize <- sprintf("%.2f %s%s", fileSize, sep, unit);
+  fileSize <- gsub(".00 bytes", " bytes", fileSize, fixed=TRUE);
+
   fileSize;
 })
 
@@ -506,6 +540,7 @@ setMethodS3("sortBy", "GenericDataFileSet", function(this, by=c("lexicographic",
 ###########################################################################/**
 # @RdocMethod getNames
 # @aliasmethod getFullNames
+# @aliasmethod names
 #
 # @title "Gets the names (or fullnames) of the files in the file set"
 #
@@ -521,6 +556,8 @@ setMethodS3("sortBy", "GenericDataFileSet", function(this, by=c("lexicographic",
 # \arguments{
 #  \item{...}{Arguments passed to \code{getName()} (\code{getFullName()})
 #    of each file.}
+#  \item{onRemapping}{Action to take if the fullnames before and after
+#    translation do not map consistently to the same file indices.}
 # }
 #
 # \value{
@@ -541,12 +578,59 @@ setMethodS3("getNames", "GenericDataFileSet", function(this, ...) {
   unname(res);
 })
 
-setMethodS3("getFullNames", "GenericDataFileSet", function(this, ...) {
-  files <- as.list(this, useNames=FALSE);
-  res <- unlist(lapply(files, FUN=getFullName, ...));
-  unname(res);
+setMethodS3("getFullNames", "GenericDataFileSet", function(this, ..., onRemapping=getOption("R.filesets::onRemapping", "ignore")) {
+  ## Argument 'onRemapping':
+  onRemapping <- match.arg(onRemapping, choices=c("ignore", "warning", "error"))
+
+  files <- as.list(this, useNames=TRUE, translate=FALSE)
+  names <- unlist(lapply(files, FUN=getFullName, ...), use.names=FALSE)
+
+  ## Assert bijective mapping after translation?
+  if (onRemapping != "ignore" && length(names) > 1L) {
+    names0 <- names(files)
+    idxs0 <- match(unique(names0), names0)
+    idxs <- match(unique(names), names)
+    if (!identical(idxs, idxs0)) {
+      signal <- if (onRemapping == "warning") warning else throw;
+
+      msg <- sprintf("%s %s: Invalid full-names translation detected. One or more of the full-names translator functions need to be corrected.", class(this)[1], sQuote(getFullName(this)))
+
+      missing <- setdiff(idxs0, idxs)
+      if (length(missing) > 0L) {
+        map <- sprintf("%s->%s used to maps to #%d", sQuote(names0[missing]), sQuote(names[missing]), missing)
+        msg <- sprintf("%s After translation, some names no longer map to an index (%s).", msg, hpaste(map, collapse="; "))
+      }
+
+      ## NB: Can this even happen?
+      extra <- setdiff(idxs, idxs0)
+      if (length(extra) > 0L) {
+        map <- sprintf("%s->%s now to maps to #%d", sQuote(names0[extra]), sQuote(names[extra]), extra)
+        msg <- sprintf("%s After translation, some names map to previously unknown indices (%s).", msg, hpaste(map, collapse="; "))
+      }
+
+      ## Otherwise...
+      if (length(missing) == 0L && length(extra) == 0L) {
+        neq <- (idxs != idxs0)
+        names <- names[neq]
+        idxs <- idxs[neq]
+        names0 <- names0[neq]
+        idxs0 <- idxs0[neq]
+        map <- sprintf("%s->#%d", names, idxs)
+        map0 <- sprintf("%s->#%d", names0, idxs0)
+        msg <- sprintf("%s The translated names has a different mapping than the non-translated ones: (%s) != (%s).", msg, hpaste(map, collapse="; "), hpaste(map0, collapse="; "))
+      }
+
+      signal(msg)
+    } # if (!identical(idxs, idxs0))
+  } ## if (validate)
+
+  names
 })
 
+setMethodS3("names", "GenericDataFileSet", function(x, ...) {
+  getFullNames(x, ...)
+}, protected=TRUE)
+
 
 
 ###########################################################################/**
@@ -2257,6 +2341,12 @@ setMethodS3("setFullNamesTranslator", "GenericDataFileSet", function(this, ...)
 
 ############################################################################
 # HISTORY:
+# 2015-05-13
+# o Added getFullNames(..., onRemapping=...) to GenericDataFileSet to
+#   warn/err on full-name translations that generates inconsistent
+#   fullname-to-index maps before and after.
+# 2015-05-12
+# o Added names() for GenericDataFileSet.  Currently returns full names.
 # 2014-08-26
 # o Added support for sortBy(..., by="filesize") and
 #   sortBy(..., decreasing=TRUE) for GenericDataFileSet.  Also, sortBy()
